@@ -6,6 +6,36 @@ import json
 
 config = Configuration()
 
+subst = {
+    'frac': [{
+        '-': "\\frac",
+        'above': '',
+    }],
+    'below': [{
+        'below': ''
+    }],
+    'sqrt': [{
+        'sqrt': '\\sqrt'
+    }],
+    'super': [{
+        'super': '^'
+    }],
+    'mult': [{
+        '*': '\cdot'
+    }],
+    'subsc': [{
+        'subsc': '',
+        '{': '',
+        '.': '.',
+        '}': ''
+    },{
+        'subsc': '_'
+    }],
+    'neq': [{
+        'neq': '\\neq'
+    }]
+}
+
 def criar_diretorio(name):
     dirname = name
     if not os.path.exists(dirname):
@@ -44,8 +74,9 @@ def save_json_file(path, data):
 
 def debug(data):
     conf = config.get_configs()
-    if conf["application"]["debug_mode"] == "active":
-        print(data)
+    if conf and "application" in conf:
+        if conf["application"]["debug_mode"] == "active":
+            print(data)
 
 
 
