@@ -15,13 +15,13 @@ class Parser:
         self.tlist=None
 
     def to_parse(self):
+        helpers.debug('[parser.py] to_parse()')
         try:
             tree = self.__main_parsing()
             self.tree=tree
 
             tlist = self.__tree_to_list(tree)
             self.tlist = tlist
-
             latex_before_cg = self.__list_to_latex_obj(tlist)
 
             check_grammar = cg.CheckGrammar()
@@ -45,6 +45,7 @@ class Parser:
             raise e
 
     def __main_parsing(self):
+        helpers.debug('\n[parser.py] __main_parsing()')
         listin = self.symbols
         T = DS.Tree()
         Q = DS.Queue()
@@ -55,6 +56,8 @@ class Parser:
 
         R = [[0,0], [9999999999, 9999999999]]
         sstart = self.__sp(listin, R)
+        helpers.debug('[parser.py] __main_parsing() | starting symbol index: %d ' % sstart)
+        helpers.debug('[parser.py] __main_parsing() | starting symbol label: %s ' % listin[sstart]['label'])
 
         s = listin[sstart]
 
@@ -62,6 +65,8 @@ class Parser:
             [s['wall']['left'], s['wall']['top']],
             [s['wall']['right'], s['wall']['bottom']]
         ]
+        helpers.debug('[parser.py] __main_parsing() | first region [R]:')
+        helpers.debug(R)
 
         if sstart != -1:
 
@@ -71,6 +76,7 @@ class Parser:
 
         while not Q.is_empty():
 
+            helpers.debug('\n[parser.py] __main_parsing() | find main baseline')
             while not Q.is_empty():
                 temp1 = Q.dequeue()
                 ParentNode = Q.dequeue()
@@ -93,6 +99,7 @@ class Parser:
                     temp2 = self.__hor(listin, temp1)
             S.push("EOBL")
 
+            helpers.debug('\n[parser.py] __main_parsing() | find secondary baseline')
             while not S.is_empty():
                 if S.peek() == "EOBL":
                     S.pop()
@@ -102,6 +109,8 @@ class Parser:
                 upperThreshold = listin[temp1]['ymin'] + ((1/6) * listin[temp1]['h'])
                 lowerThreshold = listin[temp1]['ymin'] + ((5/6) * listin[temp1]['h'])
 
+                helpers.debug('[parser.py] __main_parsing() | temp1 label: %s' % listin[temp1]['label'])
+
                 R = [
                     {'above': [
                         [listin[temp1]['xmin'], listin[temp1]['wall']['top']],
@@ -113,7 +122,11 @@ class Parser:
                     ]}
                 ]
 
+                helpers.debug('\n[parser.py] __main_parsing() | regions 1 [R]:')
+                print(R)
+
                 for region in R:
+                    # Para cada região, busca o símbolo inicial
                     reg = region[list(region.keys())[0]]
                     temp2 = self.__start(listin, reg)
 
@@ -130,18 +143,26 @@ class Parser:
                             Q.enqueue(temp2)
                             Q.enqueue(RelationNode)
 
+                helpers.debug('\n[parser.py] __main_parsing() | regions 2 [R]:')
+                print(R)
+
                 R = [
                     {'super': [
-                        [listin[temp1]['xmax'], listin[temp1]['wall']['top']],
-                        [listin[temp1]['wall']['right'], upperThreshold]
+                        [listin[temp1]['xmax'], listin[temp1]['wall']['top']], # left, top
+                        [listin[temp1]['wall']['right'], upperThreshold] # right, bottom
                     ]},
                     {'contains': [
-                        [listin[temp1]['xmin'], listin[temp1]['ymin']],
-                        [listin[temp1]['xmax'], listin[temp1]['ymax']]
+                        [listin[temp1]['xmin'], listin[temp1]['ymin']], # left, top
+                        [listin[temp1]['xmax'], listin[temp1]['ymax']] # right, bottom
                     ]},
+                    {'subsc': [
+                        [listin[temp1]['xmax'], lowerThreshold], # left, top
+                        [listin[temp1]['wall']['right'], listin[temp1]['wall']['bottom']] # right, bottom
+                    ]}
                 ]
 
                 for region in R:
+                    # Para cada região, busca o símbolo inicial
                     reg = region[list(region.keys())[0]]
                     temp2 = self.__start(listin, reg)
 
@@ -161,6 +182,7 @@ class Parser:
         return T
 
     def preprocessing(self, symbols):
+        helpers.debug('[parser.py] preprocessing()')
         xmin_sorted = sorted(symbols, key = lambda i: i['xmin'])
         symbols = xmin_sorted
 
@@ -176,10 +198,11 @@ class Parser:
             else:
                 s['type'] = 'Normal'
 
-            if re.search("^[0-9]$", str(s['label']) or s['label'] == '20'):
+            if re.search("^[0-9]$", str(s['label']) or s['label'] == '20'): # label 20 = b
                 s['centroid_class'] = 'Ascending'
                 s['centroid'][1] = s['ymin'] + (3/5) * (s['ymax'] - s['ymin'])
-            elif s['label'] == 28 or s['type'] == 'Open' or s['label'] == '25':
+                # s['centroid'][1] = s['ymin'] + (3/4) * (s['ymax'] - s['ymin'])
+            elif s['label'] == '26' or s['type'] == 'Open' or s['label'] == '24':
                 s['centroid_class'] = 'Descending'
                 s['centroid'][1] = s['ymin'] + (1/4) * s['h']
             else:
@@ -195,16 +218,22 @@ class Parser:
         return symbols
 
     def __overlap(self, symbolIndex, top, bottom, listin):
+        helpers.debug('[parser.py] __overlap()')
         listIndex = symbolIndex
         stop = False
         n = len(listin)
 
+        helpers.debug('[parser.py] __overlap() | listIndex: %d ' % listIndex)
+
         if listin[symbolIndex]['label'] == '11':
             maxLength = listin[symbolIndex]['xmax'] - listin[symbolIndex]['xmin']
         else:
             maxLength = -1
         mainLine = -1
 
+        helpers.debug('[parser.py] __overlap() | mainLine: %d ' % mainLine)
+        helpers.debug('[parser.py] __overlap() | maxLength: %d ' % maxLength)
+
         while listIndex > 0 and stop == False:
             if listin[listIndex-1]['xmin'] < listin[symbolIndex]['xmin']:
                 stop = True
@@ -216,11 +245,12 @@ class Parser:
             if not listin[listIndex]['checked'] and \
                 listin[listIndex]['label'] == '11' and \
                 listin[listIndex]['centroid'][1] >= top and \
-                listin[listIndex]['centroid'][1] < bottom and \
+                listin[listIndex]['centroid'][1] <= bottom and \
                 listin[listIndex]['xmin'] <= (listin[symbolIndex]['xmin'] + 8) and \
                 (listin[listIndex]['xmax'] - listin[listIndex]['xmin']) > maxLength:
                     maxLength = (listin[listIndex]['xmax'] - listin[listIndex]['xmin'])
                     mainLine = listIndex
+
             listIndex += 1
 
         if mainLine == -1:
@@ -229,10 +259,19 @@ class Parser:
             return mainLine
 
     def __start(self, listin, R):
-        left = R[0][0]
-        top = R[0][1]
-        right = R[1][0]
-        bottom = R[1][1]
+        helpers.debug('\n[parser.py] __start()')
+        helpers.debug('[parser.py] region [R]: ')
+        helpers.debug(R)
+
+        # Adicionei o round()
+        left = round(R[0][0])
+        top = round(R[0][1])
+        right = round(R[1][0])
+        bottom = round(R[1][1])
+
+        helpers.debug('[parser.py] rounded region [R]: left: %d, right: %d, top: %d, bottom: %d' %
+            (left, right, top, bottom)
+        )
 
         leftmostIndex = -1
         listIndex = 0
@@ -240,24 +279,38 @@ class Parser:
         n = len(listin)
 
         while leftmostIndex == -1 and listIndex < n:
+            helpers.debug('[parser.py] ... symbol index: %d' % listIndex)
+            helpers.debug('[parser.py] ... symbol label: %s' % listin[listIndex]['label'])
+            helpers.debug('[parser.py] ... symbol centroid [R]: ')
+            print(listin[listIndex]['centroid'])
+            helpers.debug('[parser.py] ... symbol rounded centroid [R]: ')
+            print(round(listin[listIndex]['centroid'][0]))
+            print(round(listin[listIndex]['centroid'][1]))
+
+            # Adicionei o round()
+            # Adicionei o >=, <=,... - era >, <,...
             if not listin[listIndex]['checked'] and \
-                listin[listIndex]['centroid'][0] > left and \
-                listin[listIndex]['centroid'][1] > top and \
-                listin[listIndex]['centroid'][0] < right and \
-                listin[listIndex]['centroid'][1] < bottom:
+                round(listin[listIndex]['centroid'][0]) >= left and \
+                round(listin[listIndex]['centroid'][1]) >= top and \
+                round(listin[listIndex]['centroid'][0]) <= right and \
+                round(listin[listIndex]['centroid'][1]) <= bottom:
                     leftmostIndex = listIndex
             else:
                 listIndex = listIndex + 1
 
+        helpers.debug('[parser.py] __start() | leftmostIndex: %d' % leftmostIndex)
+
         if leftmostIndex == -1:
             return leftmostIndex
         else:
             return self.__overlap(leftmostIndex, top, bottom, listin)
 
     def __sp(self, listin, R):
+        helpers.debug('\n[parser.py] __sp()')
         return self.__start(listin, R)
 
     def __hor(self, listin, index):
+        helpers.debug('\n[parser.py] __hor()')
         if listin[index]['type'] == 'Open':
             left = listin[index]['wall']['left']
         else:
@@ -265,13 +318,26 @@ class Parser:
 
         right = listin[index]['wall']['right']
 
-        if listin[index]['label'] == '11':
+        # Solve the region with "=", "-", "." label 
+        if int(listin[index]['label']) in [10, 11, 29]:
             top = listin[index]['wall']['top']
             bottom = listin[index]['wall']['bottom']
         else:
+            # top = listin[index]['ymin'] + (listin[index]['h'] * (1/5.9))
             top = listin[index]['ymin'] + (listin[index]['h'] * (1/6))
             bottom = listin[index]['ymin'] + (listin[index]['h'] * (5/6))
+            # bottom = listin[index]['ymin'] + (listin[index]['h'] * (5/5.9))
 
+        helpers.debug('[parser.py] __hor() current symbol label: %s' % listin[index]['label'])
+        helpers.debug('[parser.py] __hor() symbol coordinates| xmin: %d, ymin: %d, xmax: %d, ymax: %d' % 
+            (
+                listin[index]['xmin'],
+                listin[index]['ymin'],
+                listin[index]['xmax'],
+                listin[index]['ymax']
+            )
+        )
+        helpers.debug('[parser.py] __hor() | top: %d, bottom: %d, left: %d, right: %d' % (top, bottom, left, right))
         stop = False
 
         c = 0
@@ -283,25 +349,40 @@ class Parser:
                 checked = listin[s]['checked']
                 if not checked:
                     symbol = listin[s]
+
+                    helpers.debug('[parser.py] __hor() | ... symbol: %s' % symbol['label'])
+
                     if int(listin[index]['label']) in range(11,18):
+                        helpers.debug('[parser.py] __hor() | ...... before call start (11,18)')
                         R = [[left, top], [right, bottom]]
                         a = self.__start(listin, R)
                         stop = True
                     else: 
-                        if symbol['centroid'][0] > left and \
-                            symbol['centroid'][0] < right and \
-                            symbol['centroid'][1] < bottom and \
-                            symbol['centroid'][1] > top:
+                        helpers.debug('[parser.py] __hor() | ...... not(11,18)')
+                        helpers.debug('[parser.py] __hor() | ...... symbol centroid: ')
+                        print(symbol['centroid'])
+                        # Adicionei o round() nos dois
+                        # Adicionei o >=, <=, ... - era só >, <, ...
+                        if round(symbol['centroid'][0]) >= round(left) and \
+                            round(symbol['centroid'][0]) <= round(right) and \
+                            round(symbol['centroid'][1]) <= round(bottom) and \
+                            round(symbol['centroid'][1]) >= round(top):
                                 a = s
                                 stop = True
 
+        if a:
+            helpers.debug('[parser.py] __hor() | a: %d ' % a)
+            helpers.debug('[parser.py] __hor() | a label: %s ' % listin[a]['label'])
+            helpers.debug('[parser.py] __hor() | stop: %s ' % stop)
+
         if stop and a != -1:
+            helpers.debug('[parser.py] __hor() | ... before overlap')
             return self.__overlap(a, listin[a]['wall']['top'], listin[a]['wall']['bottom'], listin)
         else:
             return -1
 
     def __tree_to_list(self, tree, node=None):
-
+        helpers.debug('[parser.py] __tree_to_list()')
         latex = []
 
         def recur(root_node):
@@ -321,15 +402,15 @@ class Parser:
                         current.data['label'] = real_label
                     latex.append(current.data)
                 except BaseException as e:
-                    print('Exception b: ', e)
+                    print('Exception: ', e)
 
-            if current.node_type == 'RegionNode' or current.data['label'] == '25':
+            if current.node_type == 'RegionNode' or current.data['label'] == '24':
                 latex.append('{')
 
             for node in current.children:
                 recur(node)
 
-            if current.node_type == 'RegionNode' or current.data['label'] == '25':
+            if current.node_type == 'RegionNode' or current.data['label'] == '24':
                 latex.append('}')
 
         recur(node)
@@ -346,19 +427,33 @@ class Parser:
         return latex
 
     def __list_to_latex_obj(self, tlist):
+        helpers.debug('[parser.py] __list_to_latex_obj()')
         '''
+        Return: List of dict.
+
         GARAIN, U.; CHAUDHURI, B. B. Recognition of Online Handwritten Mathematical Expressions. IEEE Transactions on Systems, Man, and Cybernetics, [s. l.], v. 34, n. 6, p. 2366–2376, 2004. 
 
         S -> ES | E
         E -> S^{S} | S_{S} | \\frac{S}{S} | \sqrt{S} | N | L | e 
         '''
+        helpers.debug('[parser.py] __list_to_latex_obj() | tlist: ')
+        teste = []
+        for t in tlist:
+            if not isinstance(t, str):
+                del t['image']
+                teste.append(t['label'])
+            else:
+                teste.append(t)
+            print(t)
+
+        abacate = "".join(teste)
+        print('ABACATE: ', abacate)
 
         latex = []
 
         for symbol in tlist:
             if isinstance(symbol, dict):
                 latex.append({
-                    # 'label': int(symbol['label']) if symbol['label'] in numbers else symbol['label'],
                     'label': symbol['label'],
                     'prediction': symbol['prediction'] if 'prediction' in symbol else [],
                     'type': symbol['type'] or ''
@@ -376,34 +471,61 @@ class Parser:
             'sqrt': 'sqrt',
             'super': 'super',
             '*': 'mult',
+            'subsc':  'subsc',
+            'neq': 'neq'
         }
 
-        subst = {
-            'frac': {
-                '-': "\\frac",
-                'above': '',
-            },
-            'below': {
-                'below': ''
-            },
-            'sqrt': {
-                'sqrt': '\\sqrt'
-            },
-            'super': {
-                'super': '^'
-            },
-            'mult': {
-                '*': '\cdot'
-            }
-        }
+        subst = helpers.subst
 
+        eostring = len(latex)
         for i in range(0, len(latex)):
             if latex[i]['label'] in grammar:
-                subs = grammar[latex[i]['label']]
-                for substitution in subst[subs]:
-                    if latex[i]['label'] == substitution:
-                        latex[i]['label'] = subst[subs][substitution]
-                        i += 1
+                label = grammar[latex[i]['label']]
+
+                if label in subst:
+                    substitution_list = subst[label] # list of substitutions
+                    helpers.debug('[parser.py] __list_to_latex_obj() | substitution_list: ')
+                    helpers.debug(substitution_list)
+
+                    for substitution_index in range(0, len(substitution_list)):
+                        nomatch=False
+                        aux = []
+
+                        helpers.debug('[parser.py] __list_to_latex_obj() | ...substitutions: ')
+                        helpers.debug(substitution_list[substitution_index])
+
+                        initial_index = i # 9
+                        for substitution in substitution_list[substitution_index]:
+                            helpers.debug('[parser.py] __list_to_latex_obj() | ......substitution: ') # subsc
+                            helpers.debug('[parser.py] __list_to_latex_obj() | ......latex index: %d ' % i) # subsc
+                            helpers.debug(substitution) # subsc
+                            try:
+                                helpers.debug('[parser.py] __list_to_latex_obj() | ......current latex: %s ' % latex[i]['label'])
+                                if latex[i]['label'] == substitution: # latex[i]['label'] = subst[subs][substitution]
+                                    helpers.debug('[parser.py] __list_to_latex_obj() | ......match ')
+                                    helpers.debug('[parser.py] __list_to_latex_obj() | ......from: %s %s ' % (latex[i]['label'], substitution))
+                                    helpers.debug('[parser.py] __list_to_latex_obj() | ......to: %s ' % substitution_list[substitution_index][substitution])
+                                    aux.append({"index": i, "label": substitution_list[substitution_index][substitution]})
+                                    i += 1
+                                else:
+                                    helpers.debug('[parser.py] __list_to_latex_obj() | ......no match ')
+                                    i -= 1
+                                    nomatch=True
+                            except IndexError as e:
+                                helpers.debug('[parser.py] __list_to_latex_obj() | ......no match: IndexError ')
+                                nomatch=True
+                                break
+
+                            helpers.debug('[parser.py] __list_to_latex_obj() | ...... no match value: %s ' % nomatch)
+                            if nomatch:
+                                i = initial_index
+                                helpers.debug('[parser.py] __list_to_latex_obj() | ...... continue ......')
+                                break
+                            helpers.debug('[parser.py] __list_to_latex_obj() | ...... next ......')
+                        if not nomatch:
+                            helpers.debug('[parser.py] __list_to_latex_obj() | match - updating value ')
+                            for matched in aux:
+                                latex[matched['index']]['label'] = matched['label']
 
         return latex
 
